@@ -184,8 +184,8 @@ static int uwsgi_alarm_log_add(char *alarms, char *regexp, int negate) {
 
 	// map instances to the log
 	char *list = uwsgi_str(alarms);
-	char *p = strtok(list, ",");
-	while (p) {
+	char *p, *ctx = NULL;
+	uwsgi_foreach_token(list, ",", p, ctx) {
 		struct uwsgi_alarm_instance *uai = uwsgi_alarm_get_instance(p);
 		if (!uai) {
 			free(list);
@@ -205,7 +205,6 @@ static int uwsgi_alarm_log_add(char *alarms, char *regexp, int negate) {
 		else {
 			ual->alarms = uall;
 		}
-		p = strtok(NULL, ",");
 	}
 	free(list);
 	return 0;
