@@ -172,6 +172,16 @@ int uwsgi_buffer_u32be(struct uwsgi_buffer *ub, uint32_t num) {
         return uwsgi_buffer_append(ub, (char *) buf, 4);
 }
 
+int uwsgi_buffer_f32be(struct uwsgi_buffer *ub, float num) {
+        uint8_t buf[4];
+	uint32_t *pnum = (uint32_t *) &num;
+        buf[3] = (uint8_t) (*pnum & 0xff);
+        buf[2] = (uint8_t) ((*pnum >> 8) & 0xff);
+        buf[1] = (uint8_t) ((*pnum >> 16) & 0xff);
+        buf[0] = (uint8_t) ((*pnum >> 24) & 0xff);
+        return uwsgi_buffer_append(ub, (char *) buf, 4);
+}
+
 int uwsgi_buffer_u32le(struct uwsgi_buffer *ub, uint32_t num) {
         uint8_t buf[4];
         buf[0] = (uint8_t) (num & 0xff);
@@ -195,6 +205,20 @@ int uwsgi_buffer_u64be(struct uwsgi_buffer *ub, uint64_t num) {
 }
 
 
+int uwsgi_buffer_f64be(struct uwsgi_buffer *ub, double num) {
+        uint8_t buf[8];
+	uint64_t *pnum = (uint64_t *) &num;
+        buf[7] = (uint8_t) (*pnum & 0xff);
+        buf[6] = (uint8_t) ((*pnum >> 8) & 0xff);
+        buf[5] = (uint8_t) ((*pnum >> 16) & 0xff);
+        buf[4] = (uint8_t) ((*pnum >> 24) & 0xff);
+        buf[3] = (uint8_t) ((*pnum >> 32) & 0xff);
+        buf[2] = (uint8_t) ((*pnum >> 40) & 0xff);
+        buf[1] = (uint8_t) ((*pnum >> 48) & 0xff);
+        buf[0] = (uint8_t) ((*pnum >> 56) & 0xff);
+        return uwsgi_buffer_append(ub, (char *) buf, 8);
+}
+
 int uwsgi_buffer_append_ipv4(struct uwsgi_buffer *ub, void *addr) {
 	char ip[INET_ADDRSTRLEN];
 	if (!inet_ntop(AF_INET, addr, ip, INET_ADDRSTRLEN)) {
