@@ -41,7 +41,7 @@ struct uwsgi_gateway *register_gateway(char *name, void (*loop) (int, void *), v
 	uwsgi_socket_nb(ug->internal_subscription_pipe[0]);
 	uwsgi_socket_nb(ug->internal_subscription_pipe[1]);
 
-	if (!uwsgi.master_process)
+	if (!uwsgi.master_process && !uwsgi.force_gateway)
 		gateway_respawn(ushared->gateways_cnt);
 
 	ushared->gateways_cnt++;
