@@ -4,6 +4,11 @@ extern struct uwsgi_server uwsgi;
 
 // sendfile() abstraction
 ssize_t uwsgi_sendfile_do(int sockfd, int filefd, size_t pos, size_t len) {
+	// for platform not supporting sendfile we need to rely on boring read/write
+	// generally that platforms have very low memory, so use a 8k buffer
+	char buf[8192];
+
+	if (uwsgi.disable_sendfile) goto no_sendfile;
 
 #if defined(__FreeBSD__) || defined(__DragonFly__)
 	off_t sf_len = len;
@@ -18,12 +23,11 @@ ssize_t uwsgi_sendfile_do(int sockfd, int filefd, size_t pos, size_t len) {
 #elif defined(__linux__) || defined(__sun__)
 	off_t off = pos;
 	return sendfile(sockfd, filefd, &off, len);
-#else
-	// for platform not supporting sendfile we need to rely on boring read/write
-	// generally that platforms have very low memory, so use a 8k buffer
-	char buf[8192];
+#endif
+
+no_sendfile:
 	if (pos > 0) {
-		if (lseek(filefd, pos, SEEK_SET)) {
+		if (lseek(filefd, pos, SEEK_SET) < 0) {
 			uwsgi_error("uwsgi_sendfile_do()/seek()");
 			return -1;
 		}
@@ -34,7 +38,6 @@ ssize_t uwsgi_sendfile_do(int sockfd, int filefd, size_t pos, size_t len) {
 		return -1;
 	}
 	return write(sockfd, buf, rlen);
-#endif
 
 }
 
