@@ -197,14 +197,7 @@ extern "C" int uwsgi_gridfs_request(struct wsgi_request *wsgi_req) {
                 return -1;
         }
 
-        if (wsgi_req->appid_len == 0) {
-                if (!uwsgi.ignore_script_name) {
-                        wsgi_req->appid = wsgi_req->script_name;
-                        wsgi_req->appid_len = wsgi_req->script_name_len;
-                }
-        }
-
-        wsgi_req->app_id = uwsgi_get_app_id(wsgi_req->appid, wsgi_req->appid_len, gridfs_plugin.modifier1);
+        wsgi_req->app_id = uwsgi_get_app_id(wsgi_req, wsgi_req->appid, wsgi_req->appid_len, gridfs_plugin.modifier1);
         if (ugridfs.debug) {
                 uwsgi_log("[uwsgi-gridfs-debug] app_id = %d\n", wsgi_req->app_id);
         }
@@ -238,6 +231,9 @@ extern "C" int uwsgi_gridfs_request(struct wsgi_request *wsgi_req) {
 
 
 extern "C" void uwsgi_gridfs_mount() {
+	if (!uwsgi.skip_atexit) {
+		uwsgi_log("*** WARNING libmongoclient could have a bug with atexit() hooks, if you get segfault on end/reload, add --skip-atexit ***\n");
+	}
 	struct uwsgi_string_list *usl = ugridfs.mountpoints;
 	while(usl) {
 		if (uwsgi_apps_cnt >= uwsgi.max_apps) {
