@@ -181,10 +181,9 @@ static void uwsgi_opt_ldap_dump_ldif(char *opt, char *foo, void *bar) {
 		entry = &ule[i];
 
 		char *list2 = uwsgi_concat2(entry->names + 1, "");
-		char *p = strtok(list2, " ");
-		while (p != NULL) {
+		char *p, *ctx = NULL;
+		uwsgi_foreach_token(list2, " ", p, ctx) {
 			uwsgi_log("%.*s $ ", strlen(p) - 2, p + 1);
-			p = strtok(NULL, " ");
 		}
 
 		free(list2);
@@ -228,10 +227,9 @@ static void uwsgi_opt_ldap_dump(char *opt, char *foo, void *bar) {
 		entry = &ule[i];
 
 		char *list2 = uwsgi_concat2(entry->names + 1, "");
-		char *p = strtok(list2, " ");
-		while (p != NULL) {
+		char *p, *ctx = NULL;
+		uwsgi_foreach_token(list2, " ", p, ctx) {
 			uwsgi_log("%.*s $ ", strlen(p) - 2, p + 1);
-			p = strtok(NULL, " ");
 		}
 
 		free(list2);
