@@ -2,7 +2,7 @@ NAME='pypy'
 LDFLAGS = []
 LIBS = []
 GCC_LIST = ['pypy_plugin']
-BINARY_LIST = [ ('uwsgi_pypy_setup','pypy_setup.py')]
+BINARY_LIST = [ ('_uwsgi_pypy_setup','pypy_setup.py')]
 CFLAGS = []
 try:
     import __pypy__
