@@ -13,6 +13,15 @@ int PyFrame_GetLineNumber(PyFrameObject *frame) {
 }
 #endif
 
+#ifdef PYTHREE
+#undef PyString_AsString
+static char *PyString_AsString(PyObject *o) {
+	PyObject *zero = PyUnicode_AsLatin1String(o);
+	if (!zero) return "";
+        return PyBytes_AsString(zero);
+}
+#endif
+
 int uwsgi_python_profiler_call(PyObject *obj, PyFrameObject *frame, int what, PyObject *arg) {
 
 	static uint64_t last_ts = 0;
