@@ -47,5 +47,19 @@ if has_shared == 'yes':
     os.environ['LD_RUN_PATH'] = libpath
     LIBS.append(os.popen(RUBYPATH + " -e \"require 'rbconfig';print '-l' + %s::CONFIG['RUBY_SO_NAME']\"" % rbconfig).read().rstrip())
 else:
+    rubylibdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['rubylibdir']\"").read().rstrip()
+    rubyarchdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['archdir']\"").read().rstrip()
+    # detect Heroku system
+    heroku = False
+    if rubylibdir.startswith('/tmp/build_'):
+        heroku = True
+        rubylibdir = '/app/' + '/'.join(rubylibdir.split('/')[3:])
+    if rubyarchdir.startswith('/tmp/build_'):
+        heroku = True
+        rubyarchdir = '/app/' + '/'.join(rubyarchdir.split('/')[3:])
+    if heroku:
+        CFLAGS.append('-DUWSGI_RUBY_HEROKU')
+    CFLAGS.append('-DUWSGI_RUBY_LIBDIR="\\"%s\\""' % rubylibdir)
+    CFLAGS.append('-DUWSGI_RUBY_ARCHDIR="\\"%s\\""' % rubyarchdir)
     GCC_LIST.append("%s/%s" % (libpath, os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['LIBRUBY_A']\"" % rbconfig).read().rstrip()))
 
